import { Codicon } from 'vscode/vscode/vs/base/common/codicons';
import { ResourceSet } from 'vscode/vscode/vs/base/common/map';
import { basename } from 'vscode/vscode/vs/base/common/resources';
import { URI } from 'vscode/vscode/vs/base/common/uri';
import { isCodeEditor } from 'vscode/vscode/vs/editor/browser/editorBrowser';
import { localize2, localize } from 'vscode/vscode/vs/nls';
import { Action2, registerAction2, MenuId } from 'vscode/vscode/vs/platform/actions/common/actions';
import { IConfigurationService } from 'vscode/vscode/vs/platform/configuration/common/configuration.service';
import { ContextKeyExpr } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { IDialogService } from 'vscode/vscode/vs/platform/dialogs/common/dialogs.service';
import { EditorActivation } from 'vscode/vscode/vs/platform/editor/common/editor';
import { IListService } from 'vscode/vscode/vs/platform/list/browser/listService.service';
import { IEditorGroupsService } from 'vscode/vscode/vs/workbench/services/editor/common/editorGroupsService.service';
import { IEditorService } from 'vscode/vscode/vs/workbench/services/editor/common/editorService.service';
import { ChatAgentLocation } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatAgents';
import { CONTEXT_CHAT_REQUEST_IN_PROGRESS, CONTEXT_CHAT_LOCATION, CONTEXT_IN_CHAT_INPUT, CONTEXT_CHAT_INPUT_HAS_TEXT, CONTEXT_RESPONSE, CONTEXT_ITEM_ID, CONTEXT_LAST_ITEM_ID, CONTEXT_IN_CHAT_SESSION, CONTEXT_REQUEST } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatContextKeys';
import { chatEditingWidgetFileStateContextKey, CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME, chatEditingResourceContextKey, decidedChatEditingResourceContextKey, hasUndecidedChatEditingResourceContextKey, applyingChatEditsFailedContextKey, hasAppliedChatEditsContextKey, isChatRequestCheckpointed, applyingChatEditsContextKey } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/common/chatEditingService';
import { IChatEditingService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatEditingService.service';
import { IChatService } from 'vscode/vscode/vs/workbench/contrib/chat/common/chatService.service';
import { isResponseVM, isRequestVM } from '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/common/chatViewModel';
import '@codingame/monaco-vscode-chat-extensions-interactive-notebook-search-terminal-common/vscode/vs/workbench/contrib/chat/browser/actions/chatActions';
import { IChatWidgetService } from 'vscode/vscode/vs/workbench/contrib/chat/browser/chat.service';
import { CHAT_CATEGORY } from 'vscode/vscode/vs/workbench/contrib/chat/browser/actions/chatConstants';

class WorkingSetAction extends Action2 {
    run(accessor, ...args) {
        const chatEditingService = accessor.get(IChatEditingService);
        const currentEditingSession = chatEditingService.currentEditingSession;
        if (!currentEditingSession) {
            return;
        }
        const chatWidget = accessor.get(IChatWidgetService).lastFocusedWidget;
        if (chatWidget?.location !== ChatAgentLocation.EditingSession) {
            return;
        }
        const uris = [];
        if (URI.isUri(args[0])) {
            uris.push(args[0]);
        }
        else if (chatWidget) {
            uris.push(...chatWidget.input.selectedElements);
        }
        if (!uris.length) {
            return;
        }
        return this.runWorkingSetAction(accessor, currentEditingSession, chatWidget, ...uris);
    }
}
registerAction2(class RemoveFileFromWorkingSet extends WorkingSetAction {
    constructor() {
        super({
            id: 'chatEditing.removeFileFromWorkingSet',
            title: ( localize2(10804, 'Remove File')),
            icon: Codicon.close,
            menu: [{
                    id: MenuId.ChatEditingWidgetModifiedFilesToolbar,
                    when: ( (ContextKeyExpr.or( (ContextKeyExpr.equals(
                        chatEditingWidgetFileStateContextKey.key,
                        4
                    )),  (ContextKeyExpr.equals(
                        chatEditingWidgetFileStateContextKey.key,
                        3
                    ))))),
                    order: 0,
                    group: 'navigation'
                }],
        });
    }
    async runWorkingSetAction(accessor, currentEditingSession, chatWidget, ...uris) {
        currentEditingSession.remove(...uris);
        const resourceSet = ( (new ResourceSet(uris)));
        const newContext = [];
        for (const context of chatWidget.input.attachmentModel.attachments) {
            if (!URI.isUri(context.value) || !context.isFile || !( (resourceSet.has(context.value)))) {
                newContext.push(context);
            }
        }
        chatWidget.attachmentModel.clearAndSetContext(...newContext);
    }
});
registerAction2(class OpenFileInDiffAction extends WorkingSetAction {
    constructor() {
        super({
            id: 'chatEditing.openFileInDiff',
            title: ( localize2(10805, 'Open Changes in Diff Editor')),
            icon: Codicon.diffSingle,
            menu: [{
                    id: MenuId.ChatEditingWidgetModifiedFilesToolbar,
                    when: ( (ContextKeyExpr.equals(
                        chatEditingWidgetFileStateContextKey.key,
                        0
                    ))),
                    order: 2,
                    group: 'navigation'
                }],
        });
    }
    async runWorkingSetAction(accessor, currentEditingSession, _chatWidget, ...uris) {
        const editorService = accessor.get(IEditorService);
        for (const uri of uris) {
            const editedFile = currentEditingSession.entries.get().find((e) => ( (e.modifiedURI.toString())) === ( (uri.toString())));
            if (editedFile?.state.get() === 0 ) {
                await editorService.openEditor({
                    original: { resource: ( (URI.from(editedFile.originalURI, true))) },
                    modified: { resource: ( (URI.from(editedFile.modifiedURI, true))) },
                });
            }
            else {
                await editorService.openEditor({ resource: uri });
            }
        }
    }
});
registerAction2(class AcceptAction extends WorkingSetAction {
    constructor() {
        super({
            id: 'chatEditing.acceptFile',
            title: ( localize2(10806, 'Accept')),
            icon: Codicon.check,
            menu: [{
                    when: ( (ContextKeyExpr.and(
                         (ContextKeyExpr.equals('resourceScheme', CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME)),
                        ContextKeyExpr.notIn(chatEditingResourceContextKey.key, decidedChatEditingResourceContextKey.key)
                    ))),
                    id: MenuId.MultiDiffEditorFileToolbar,
                    order: 0,
                    group: 'navigation',
                }, {
                    id: MenuId.ChatEditingWidgetModifiedFilesToolbar,
                    when: ( (ContextKeyExpr.equals(
                        chatEditingWidgetFileStateContextKey.key,
                        0
                    ))),
                    order: 0,
                    group: 'navigation'
                }],
        });
    }
    async runWorkingSetAction(accessor, currentEditingSession, chatWidget, ...uris) {
        await currentEditingSession.accept(...uris);
    }
});
registerAction2(class DiscardAction extends WorkingSetAction {
    constructor() {
        super({
            id: 'chatEditing.discardFile',
            title: ( localize2(10807, 'Discard')),
            icon: Codicon.discard,
            menu: [{
                    when: ( (ContextKeyExpr.and(
                         (ContextKeyExpr.equals('resourceScheme', CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME)),
                        ContextKeyExpr.notIn(chatEditingResourceContextKey.key, decidedChatEditingResourceContextKey.key)
                    ))),
                    id: MenuId.MultiDiffEditorFileToolbar,
                    order: 2,
                    group: 'navigation',
                }, {
                    id: MenuId.ChatEditingWidgetModifiedFilesToolbar,
                    when: ( (ContextKeyExpr.equals(
                        chatEditingWidgetFileStateContextKey.key,
                        0
                    ))),
                    order: 1,
                    group: 'navigation'
                }],
        });
    }
    async runWorkingSetAction(accessor, currentEditingSession, chatWidget, ...uris) {
        await currentEditingSession.reject(...uris);
    }
});
class ChatEditingAcceptAllAction extends Action2 {
    constructor() {
        super({
            id: 'chatEditing.acceptAllFiles',
            title: ( localize(10808, 'Accept')),
            icon: Codicon.check,
            tooltip: ( localize(10809, 'Accept All Edits')),
            precondition: ( (ContextKeyExpr.and(
                 (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate()),
                hasUndecidedChatEditingResourceContextKey
            ))),
            keybinding: {
                primary: 2048  | 3 ,
                when: ( (ContextKeyExpr.and(
                     (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate()),
                    hasUndecidedChatEditingResourceContextKey,
                     (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession)),
                    CONTEXT_IN_CHAT_INPUT
                ))),
                weight: 200 ,
            },
            menu: [
                {
                    when: ( (ContextKeyExpr.equals('resourceScheme', CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME))),
                    id: MenuId.EditorTitle,
                    order: 0,
                    group: 'navigation',
                },
                {
                    id: MenuId.ChatEditingWidgetToolbar,
                    group: 'navigation',
                    order: 0,
                    when: ( (ContextKeyExpr.and(
                         (applyingChatEditsFailedContextKey.negate()),
                         (ContextKeyExpr.or(
                             (hasAppliedChatEditsContextKey.negate()),
                             (ContextKeyExpr.and(
                                hasUndecidedChatEditingResourceContextKey,
                                 (ContextKeyExpr.and(
                                     (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession))
                                ))
                            ))
                        ))
                    )))
                }
            ]
        });
    }
    async run(accessor, ...args) {
        const chatEditingService = accessor.get(IChatEditingService);
        const currentEditingSession = chatEditingService.currentEditingSession;
        if (!currentEditingSession) {
            return;
        }
        await currentEditingSession.accept();
    }
}
registerAction2(ChatEditingAcceptAllAction);
class ChatEditingDiscardAllAction extends Action2 {
    constructor() {
        super({
            id: 'chatEditing.discardAllFiles',
            title: ( localize(10810, 'Discard')),
            icon: Codicon.discard,
            tooltip: ( localize(10811, 'Discard All Edits')),
            precondition: ( (ContextKeyExpr.and(
                 (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate()),
                hasUndecidedChatEditingResourceContextKey
            ))),
            menu: [
                {
                    when: ( (ContextKeyExpr.equals('resourceScheme', CHAT_EDITING_MULTI_DIFF_SOURCE_RESOLVER_SCHEME))),
                    id: MenuId.EditorTitle,
                    order: 1,
                    group: 'navigation',
                },
                {
                    id: MenuId.ChatEditingWidgetToolbar,
                    group: 'navigation',
                    order: 1,
                    when: ( (ContextKeyExpr.and(
                         (applyingChatEditsFailedContextKey.negate()),
                         (ContextKeyExpr.or(
                             (hasAppliedChatEditsContextKey.negate()),
                             (ContextKeyExpr.and(
                                 (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession)),
                                hasUndecidedChatEditingResourceContextKey
                            ))
                        ))
                    )))
                }
            ],
            keybinding: {
                when: ( (ContextKeyExpr.and(
                     (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate()),
                    hasUndecidedChatEditingResourceContextKey,
                     (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession)),
                    CONTEXT_IN_CHAT_INPUT,
                     (CONTEXT_CHAT_INPUT_HAS_TEXT.negate())
                ))),
                weight: 200 ,
                primary: 2048  | 1 ,
            },
        });
    }
    async run(accessor, ...args) {
        const chatEditingService = accessor.get(IChatEditingService);
        const currentEditingSession = chatEditingService.currentEditingSession;
        if (!currentEditingSession) {
            return;
        }
        await currentEditingSession.reject();
    }
}
registerAction2(ChatEditingDiscardAllAction);
class ChatEditingShowChangesAction extends Action2 {
    static { this.ID = 'chatEditing.viewChanges'; }
    static { this.LABEL = ( localize(10812, 'View All Edits')); }
    constructor() {
        super({
            id: ChatEditingShowChangesAction.ID,
            title: ChatEditingShowChangesAction.LABEL,
            tooltip: ChatEditingShowChangesAction.LABEL,
            f1: false,
            icon: Codicon.diffMultiple,
            precondition: hasUndecidedChatEditingResourceContextKey,
            menu: [
                {
                    id: MenuId.ChatEditingWidgetToolbar,
                    group: 'navigation',
                    order: 4,
                    when: ( (ContextKeyExpr.and(
                         (applyingChatEditsFailedContextKey.negate()),
                         (ContextKeyExpr.or(
                             (hasAppliedChatEditsContextKey.negate()),
                             (ContextKeyExpr.and(
                                hasAppliedChatEditsContextKey,
                                hasUndecidedChatEditingResourceContextKey,
                                 (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession))
                            ))
                        ))
                    )))
                }
            ],
        });
    }
    async run(accessor, ...args) {
        const chatEditingService = accessor.get(IChatEditingService);
        const currentEditingSession = chatEditingService.currentEditingSession;
        if (!currentEditingSession) {
            return;
        }
        await currentEditingSession.show();
    }
}
registerAction2(ChatEditingShowChangesAction);
registerAction2(class AddFilesToWorkingSetAction extends Action2 {
    constructor() {
        super({
            id: 'workbench.action.chat.addSelectedFilesToWorkingSet',
            title: ( localize2(10813, "Add Selected Files to Working Set")),
            icon: Codicon.attach,
            category: CHAT_CATEGORY,
            precondition: ( (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession))),
            f1: true
        });
    }
    async run(accessor, ...args) {
        const listService = accessor.get(IListService);
        const chatEditingService = accessor.get(IChatEditingService);
        const editorGroupService = accessor.get(IEditorGroupsService);
        const uris = [];
        for (const group of editorGroupService.getGroups(1 )) {
            for (const selection of group.selectedEditors) {
                if (selection.resource) {
                    uris.push(selection.resource);
                }
            }
        }
        if (uris.length === 0) {
            const selection = listService.lastFocusedList?.getSelection();
            if (selection?.length) {
                for (const file of selection) {
                    if (!!file && typeof file === 'object' && 'resource' in file && URI.isUri(file.resource)) {
                        uris.push(file.resource);
                    }
                }
            }
        }
        for (const file of uris) {
            chatEditingService?.currentEditingSessionObs.get()?.addFileToWorkingSet(file);
        }
    }
});
registerAction2(class RestoreWorkingSetAction extends Action2 {
    constructor() {
        super({
            id: 'workbench.action.chat.restoreFile',
            title: ( localize2(10814, 'Restore File Snapshot')),
            f1: false,
            icon: Codicon.target,
            shortTitle: ( localize2(10815, 'Restore Snapshot')),
            toggled: {
                condition: isChatRequestCheckpointed,
                title: ( localize2(10816, 'Using Snapshot')).value,
                tooltip: ( localize(
                    10817,
                    'Toggle to use a previous snapshot of an edited file in your next request'
                ))
            },
            precondition: ( (ContextKeyExpr.and(
                 (applyingChatEditsContextKey.negate()),
                 (CONTEXT_CHAT_REQUEST_IN_PROGRESS.negate())
            ))),
            menu: {
                id: MenuId.ChatMessageFooter,
                group: 'navigation',
                order: 1000,
                when: ( (ContextKeyExpr.and(
                     (ContextKeyExpr.equals('config.chat.editing.experimental.enableRestoreFile', true)),
                     (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession)),
                    CONTEXT_RESPONSE,
                    ContextKeyExpr.notIn(CONTEXT_ITEM_ID.key, CONTEXT_LAST_ITEM_ID.key)
                )))
            }
        });
    }
    run(accessor, ...args) {
        const chatEditingService = accessor.get(IChatEditingService);
        const item = args[0];
        if (!isResponseVM(item)) {
            return;
        }
        const { session, requestId } = item.model;
        const shouldUnsetCheckpoint = requestId === session.checkpoint?.id;
        if (shouldUnsetCheckpoint) {
            session.setCheckpoint(undefined);
        }
        else {
            session.setCheckpoint(requestId);
        }
        const chatService = accessor.get(IChatService);
        const chatModel = chatService.getSession(item.sessionId);
        const chatRequests = chatModel?.getRequests();
        const snapshot = chatRequests?.find((v, i) => i > 0 && chatRequests[i - 1]?.id === requestId);
        if (!shouldUnsetCheckpoint && snapshot !== undefined) {
            chatEditingService.restoreSnapshot(snapshot.id);
        }
        else if (shouldUnsetCheckpoint) {
            chatEditingService.restoreSnapshot(undefined);
        }
    }
});
registerAction2(class RemoveAction extends Action2 {
    constructor() {
        super({
            id: 'workbench.action.chat.undoEdits',
            title: ( localize2(10818, "Undo Edits")),
            f1: false,
            category: CHAT_CATEGORY,
            icon: Codicon.x,
            keybinding: {
                primary: 20 ,
                mac: {
                    primary: 2048  | 1 ,
                },
                when: ( (ContextKeyExpr.and(
                     (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession)),
                    CONTEXT_IN_CHAT_SESSION,
                     (CONTEXT_IN_CHAT_INPUT.negate())
                ))),
                weight: 200 ,
            },
            menu: [
                {
                    id: MenuId.ChatMessageTitle,
                    group: 'navigation',
                    order: 2,
                    when: ( (ContextKeyExpr.and(
                         (CONTEXT_CHAT_LOCATION.isEqualTo(ChatAgentLocation.EditingSession)),
                        CONTEXT_REQUEST
                    )))
                }
            ]
        });
    }
    async run(accessor, ...args) {
        let item = args[0];
        if (!isResponseVM(item) && !isRequestVM(item)) {
            const chatWidgetService = accessor.get(IChatWidgetService);
            const widget = chatWidgetService.lastFocusedWidget;
            item = widget?.getFocus();
        }
        if (!item) {
            return;
        }
        const chatService = accessor.get(IChatService);
        const chatModel = chatService.getSession(item.sessionId);
        if (chatModel?.initialLocation !== ChatAgentLocation.EditingSession) {
            return;
        }
        const requestId = isRequestVM(item) ? item.id :
            isResponseVM(item) ? item.requestId : undefined;
        if (requestId) {
            const configurationService = accessor.get(IConfigurationService);
            const dialogService = accessor.get(IDialogService);
            const chatEditingService = accessor.get(IChatEditingService);
            const chatRequests = chatModel.getRequests();
            const itemIndex = chatRequests.findIndex(request => request.id === requestId);
            const editsToUndo = chatRequests.length - itemIndex;
            const requestsToRemove = chatRequests.slice(itemIndex);
            const requestIdsToRemove = ( (new Set( (requestsToRemove.map(request => request.id)))));
            const entriesModifiedInRequestsToRemove = chatEditingService.currentEditingSessionObs.get()?.entries.get().filter((entry) => ( (requestIdsToRemove.has(entry.lastModifyingRequestId)))) ?? [];
            const shouldPrompt = entriesModifiedInRequestsToRemove.length > 0 && configurationService.getValue('chat.editing.confirmEditRequestRemoval') === true;
            let message;
            if (editsToUndo === 1) {
                if (entriesModifiedInRequestsToRemove.length === 1) {
                    message = ( localize(
                        10819,
                        "This will remove your last request and undo the edits made to {0}. Do you want to proceed?",
                        basename(entriesModifiedInRequestsToRemove[0].modifiedURI)
                    ));
                }
                else {
                    message = ( localize(
                        10820,
                        "This will remove your last request and undo edits made to {0} files in your working set. Do you want to proceed?",
                        entriesModifiedInRequestsToRemove.length
                    ));
                }
            }
            else {
                if (entriesModifiedInRequestsToRemove.length === 1) {
                    message = ( localize(
                        10821,
                        "This will remove all subsequent requests and undo edits made to {0}. Do you want to proceed?",
                        basename(entriesModifiedInRequestsToRemove[0].modifiedURI)
                    ));
                }
                else {
                    message = ( localize(
                        10822,
                        "This will remove all subsequent requests and undo edits made to {0} files in your working set. Do you want to proceed?",
                        entriesModifiedInRequestsToRemove.length
                    ));
                }
            }
            const confirmation = shouldPrompt
                ? await dialogService.confirm({
                    title: editsToUndo === 1
                        ? ( localize(10823, "Do you want to undo your last edit?"))
                        : ( localize(10824, "Do you want to undo {0} edits?", editsToUndo)),
                    message: message,
                    primaryButton: ( localize(10825, "Yes")),
                    checkbox: { label: ( localize(10826, "Don't ask again")), checked: false },
                    type: 'info'
                })
                : { confirmed: true };
            if (!confirmation.confirmed) {
                return;
            }
            if (confirmation.checkboxChecked) {
                await configurationService.updateValue('chat.editing.confirmEditRequestRemoval', false);
            }
            const snapshotRequestId = chatRequests[itemIndex].id;
            await chatEditingService.restoreSnapshot(snapshotRequestId);
            for (const request of requestsToRemove) {
                await chatService.removeRequest(item.sessionId, request.id);
            }
        }
    }
});
registerAction2(class OpenWorkingSetHistoryAction extends Action2 {
    static { this.id = 'chat.openFileSnapshot'; }
    constructor() {
        super({
            id: OpenWorkingSetHistoryAction.id,
            title: ( localize(10827, "Open File Snapshot")),
            menu: [{
                    id: MenuId.ChatEditingCodeBlockContext,
                    group: 'navigation',
                    order: 0,
                    when: ContextKeyExpr.notIn(CONTEXT_ITEM_ID.key, CONTEXT_LAST_ITEM_ID.key),
                },]
        });
    }
    async run(accessor, ...args) {
        const context = args[0];
        if (!context?.sessionId) {
            return;
        }
        const chatService = accessor.get(IChatService);
        const chatEditingService = accessor.get(IChatEditingService);
        const editorService = accessor.get(IEditorService);
        const chatModel = chatService.getSession(context.sessionId);
        const requests = chatModel?.getRequests();
        if (!requests) {
            return;
        }
        const snapshotRequestIndex = requests?.findIndex((v, i) => i > 0 && requests[i - 1]?.id === context.requestId);
        if (snapshotRequestIndex < 1) {
            return;
        }
        const snapshotRequestId = requests[snapshotRequestIndex]?.id;
        if (snapshotRequestId) {
            const snapshot = chatEditingService.getSnapshotUri(snapshotRequestId, context.uri);
            if (snapshot) {
                const editor = await editorService.openEditor({ resource: snapshot, label: ( localize(
                    10828,
                    '{0} (Snapshot {1})',
                    basename(context.uri),
                    snapshotRequestIndex - 1
                )), options: { transient: true, activation: EditorActivation.ACTIVATE } });
                if (isCodeEditor(editor)) {
                    editor.updateOptions({ readOnly: true });
                }
            }
        }
    }
});

export { ChatEditingAcceptAllAction, ChatEditingDiscardAllAction, ChatEditingShowChangesAction };
